#' Mise en forme des tables ouvertes
#'
#' Extraction des donnees d'une table ouverte, simplification par tables de correspondance et mise au format data
#'
#' @param chemin chemin menant au fichier "remise" à évaluer
#' @param titre nom du fichier remise à évaluer (avec .xlsx à la fin du nom)
#' @param list_periode liste des periodes ou de la repetition des colonnes de resultats
#' @param list_scen liste des prefixe des scenarios tels que presents dans le nom des onglets de la remise
#'
#' @return opendata_flat, un tableau plat des donnees de la remise concernant la table ouverte
#'
#' @examples
#' chemin<-"./exemples_chemin_utilisateur/" # repertoire de travail où se trouve la remise à évaluer
#' titre<-"Remise_user_V1.xlsx" # nom de la remise a évaluer
#' data<-MEF_data(chemin,titre)
#' rule_checks(data)
#'
#' @author Sebastien Gallet <sebastien.gallet@acpr.banque-france.fr>
#' @export


##########################################################
MEF_opendata<- function(chemin,titre,list_periode,list_scen,nom_participant) {
  # list_periode<-list_periode
  # list_scen<-list_onglet_opendata
  # nom_participant<-nom_particip

  nb_scen<-length(list_scen) # nombre de scenario dans la remise

  ######################################################
  # import des tables de correspondances
  ######################################################
  data("CIC_tab")
  data("Country_tab")
  data("Sector_tab")
  data("RC_code")

  ######################################################
  # formation des lignes Actifs (table ouverte) a integrer à data
  ######################################################
  opendata_flat<-data.frame(t(rep("",5)),stringsAsFactors = FALSE)
  colnames(opendata_flat)<-c("nom_participant","onglet","code_ligne","code_col","valeur")
  for (sc in 1:nb_scen) { # sc=1
    onglet<-list_scen[sc]

    # lecture de la table ouverte
    # Actifs<-read.xlsx(file=paste(chemin,titre,sep=""), sheetName = onglet,startRow = 7,stringsAsFactors = FALSE, as.data.frame=TRUE, header=TRUE)
    Actifs<-read_xlsx(path=paste(chemin,titre,sep=""), sheet = onglet, skip = 6,col_names=TRUE,trim_ws=TRUE)
    Actifs<-as.data.frame(Actifs[-which(is.na(Actifs[,1])),])
    # gc()
    # modification/simplification des champs selon tables de correspondances
    ######################
    # sector C0030
    sector<-Sector_tab$Simplified.sector[match(substr(as.character(Actifs$C0030),1,1),substr(Sector_tab$Sector,1,1))]
    sector<-RC_code$R[match(sector,RC_code$C0030[which(!is.na(RC_code$C0030))])]
    sector[which(is.na(sector))]<-"R0000"
    ######################

    ######################
    # Country C0040
    # equivalence des pays actifs avec les listes prédéinie
    country<-Country_tab$Zone.geo[match(as.character(Actifs$C0040),Country_tab$Used.name)]
    # equivalence nom pays avec code R00XX
    country<-RC_code$R[match(country,RC_code$C0040[which(!is.na(RC_code$C0040))])]
    country[which(is.na(country))]<-"R0000"
    ###################### str(country)

    ######################
    # nature CIC 3ieme char C0020
    nb_char_retenu<-function (code) {
      if (!is.na(as.character(substr(code,1,1))) ) {
        if (sum(substr(code,1,1)==LETTERS[])>=1) { # c'est une lettre
          CIC<-as.character(substr(code,3,3))
        } else {
          if ( !is.na(as.numeric(substr(code,1,1))/1) ) { # c'est un chiffre
            CIC<-as.character(substr(code,1,1))
          } else { # c'est  ni une lettre ni un chiffre
            CIC<-0
          }
        }} else {CIC<-""}
      return(CIC)
    } # fonction pour lire la nature dans le code CIC
    a<-sapply(as.character(Actifs$C0020),function(x) nb_char_retenu(x)) # liste nature
    nature<-CIC_tab$CIC_lettre3[match(a,substr(CIC_tab$CIC_lettre3,1,1))] # simplification liste nature
    nature<-RC_code$R[match(nature,RC_code$C0020[which(!is.na(RC_code$C0020))])]
    nature[which(is.na(nature))]<-"R0000"
    ######################

    ######################
    # cote/non cote C0050
    cote<-as.character(Actifs$C0050  )
    cote<-RC_code$R[match(substr(cote,1,2),substr(RC_code$C0050[which(!is.na(RC_code$C0050))],1,2) )]
    cote[which(is.na(cote))]<-"R0000"
    ######################

    # mise en forme du tableau simplifie
    data<-cbind(nature,sector,country,cote,Actifs$C0060,Actifs$C0070,Actifs$C0080,Actifs$C0090,Actifs$C0100)
    colnames(data)<-c("nature","sector","country","cote","C0060","C0070","C0080","C0090","C0100")
    data<-as.data.frame(data,stringsAsFactors = FALSE)
    data$C0060<-as.numeric(data$C0060)
    data$C0070<-as.numeric(data$C0070)
    data$C0080<-as.numeric(data$C0080)
    data$C0090<-as.numeric(data$C0090)
    data$C0100<-as.numeric(data$C0100)
    data[which(is.na(data),arr.ind=TRUE)]<-0
    ###################### str(data)

    ######################
    # aggregation des doublons
    data_grouped<-data %>% group_by(nature,sector,country,cote) %>% summarise(sum(C0060),sum(C0070),sum(C0080),sum(C0090),sum(C0100)) #
    names(data_grouped)<-c("C0020","C0030","C0040","C0050","C0060","C0070","C0080","C0090","C0100")
    a<-as.data.frame(data_grouped)
    # mise en forme en tableau plat "vertical"
    a<-a %>% gather("periodes", "valeur" , 5:9)
    a<- a %>% gather("col", "lig" , 1:4)
    a<-as.data.frame(a)
    # formation de data_flat actifs
    nom_particip_list<-rep(nom_participant,length(a[,1]))
    onglet_list<-rep(onglet,length(length(a[,1])))
    data_flat<-cbind(nom_particip_list,onglet_list)
    data_flat<-cbind(data_flat,a[,"lig"], paste0(a[,"col"],"/",a[,"periodes"]) , a[,"valeur"] )
    colnames(data_flat)<-c("nom_participant","onglet","code_ligne","code_col","valeur")
    ######################################################
    opendata_flat<-rbind(opendata_flat,data_flat)
  } # fin de la boucle sur les scenario
opendata_flat<-opendata_flat[-1,]
return(opendata_flat)
}
##########################################################
